from __future__ import annotations

import traceback
from typing import Any

import click
from click.core import Command, Context

import plain.runtime
from plain.exceptions import ImproperlyConfigured

from .agent import agent
from .build import build
from .changelog import changelog
from .chores import chores
from .docs import docs
from .formatting import PlainContext
from .install import install
from .preflight import preflight_cli
from .registry import cli_registry
from .scaffold import create
from .server import server
from .settings import setting
from .shell import run, shell
from .upgrade import upgrade
from .urls import urls
from .utils import utils


@click.group()
def plain_cli() -> None:
    pass


plain_cli.add_command(agent)
plain_cli.add_command(docs)
plain_cli.add_command(preflight_cli)
plain_cli.add_command(create)
plain_cli.add_command(chores)
plain_cli.add_command(build)
plain_cli.add_command(utils)
plain_cli.add_command(urls)
plain_cli.add_command(changelog)
plain_cli.add_command(setting)
plain_cli.add_command(shell)
plain_cli.add_command(run)
plain_cli.add_command(install)
plain_cli.add_command(upgrade)
plain_cli.add_command(server)


class CLIRegistryGroup(click.Group):
    """
    Click Group that exposes commands from the CLI registry.
    """

    def __init__(self, *args: Any, **kwargs: Any):
        super().__init__(*args, **kwargs)
        cli_registry.import_modules()

    def list_commands(self, ctx: Context) -> list[str]:
        return sorted(cli_registry.get_commands().keys())

    def get_command(self, ctx: Context, name: str) -> Command | None:
        commands = cli_registry.get_commands()
        return commands.get(name)


class PlainCommandCollection(click.CommandCollection):
    context_class = PlainContext

    def __init__(self, *args: Any, **kwargs: Any):
        # Start with only built-in commands (no setup needed)
        sources = [plain_cli]

        super().__init__(*args, **kwargs)
        self.sources = sources
        self._registry_group = None
        self._setup_attempted = False

    def _ensure_registry_loaded(self) -> None:
        """Lazy load the registry group (requires setup)."""
        if self._registry_group is not None or self._setup_attempted:
            return

        self._setup_attempted = True

        try:
            plain.runtime.setup()
            self._registry_group = CLIRegistryGroup()
            # Add registry group to sources
            self.sources.insert(0, self._registry_group)
        except plain.runtime.AppPathNotFound:
            # Allow built-in commands to work regardless of being in a valid app
            click.secho(
                "Plain `app` directory not found. Some commands may be missing.",
                fg="yellow",
                err=True,
            )
        except ImproperlyConfigured as e:
            # Show what was configured incorrectly and exit
            click.secho(
                str(e),
                fg="red",
                err=True,
            )
            exit(1)
        except Exception as e:
            # Show the exception and exit
            print("---")
            print(traceback.format_exc())
            print("---")

            click.secho(
                f"Error: {e}",
                fg="red",
                err=True,
            )
            exit(1)

    def get_command(self, ctx: Context, cmd_name: str) -> Command | None:
        # Try built-in commands first
        cmd = super().get_command(ctx, cmd_name)

        if cmd is None:
            # Command not found in built-ins, try registry (requires setup)
            self._ensure_registry_loaded()
            cmd = super().get_command(ctx, cmd_name)
        elif not getattr(cmd, "without_runtime_setup", False):
            # Command found but needs setup - ensure registry is loaded
            self._ensure_registry_loaded()

        if cmd:
            # Pass the formatting down to subcommands automatically
            cmd.context_class = self.context_class
        return cmd

    def list_commands(self, ctx: Context) -> list[str]:
        # For help listing, we need to show registry commands too
        self._ensure_registry_loaded()
        return super().list_commands(ctx)


cli = PlainCommandCollection()
